@@ -10,7 +10,6 @@ from functools import wraps
 from .context_manager import ContextManager
 from .class_registry import ClassRegistry, CacheStrategy
 
-
 # Metadata entrypoint support depends on Python version
 import importlib.util
 if importlib.util.find_spec("importlib.metadata"):
@@ -111,12 +110,19 @@ class InjectionManager:
 
     def register_constructor(self, cls_name, constructor, *args, **kwargs):
         """ Wrapper around :meth:`autoinject.class_registry.ClassRegistry.register_class` """
+        clear_cache = self.cls_registry.is_injectable(cls_name)
         self.cls_registry.register(cls_name, *args, constructor=constructor, **kwargs)
+        if clear_cache:
+            self.context_manager.clear_cache(cls_name)
 
     def get(self, cls_name):
         """ Wrapper around :meth:`autoinject.context_manager.ContextManager.get_object` """
         return self.context_manager.get_object(cls_name)
 
+    def override(self, cls_name, new_constructor, *args, **kwargs):
+        """ Override one class with another. """
+        self.register_constructor(cls_name, new_constructor, *args, **kwargs)
+
     def register(self, cls_name, *args, **kwargs):
         r""" Decorator for advanced registration of injectable objects. Includes support for passing positional and
              keyword arguments to the constructor, and for specifying an alternative constructor method
@@ -141,7 +147,7 @@ class InjectionManager:
             :param kwargs: Keyword arguments for the constructor
         """
         def outer_wrap(constructor):
-            self.cls_registry.register(cls_name, *args, **kwargs, constructor=constructor)
+            self.register_constructor(cls_name, constructor, *args, **kwargs)
             return constructor
         return outer_wrap
 
@@ -160,7 +166,25 @@ class InjectionManager:
                     pass
 
         """
-        self.cls_registry.register(cls)
+        self.register_constructor(cls, None)
+        return cls
+
+    def injectable_global(self, cls):
+        """injectable_global()
+
+        Class decorator for basic registration of injectable objects that don't require external input, but with
+        a global scope.
+        """
+        self.register_constructor(cls, None, caching_strategy=CacheStrategy.GLOBAL_CACHE)
+        return cls
+
+    def injectable_nocache(self, cls):
+        """injectable_nocache()
+
+        Class decorator for basic registration of injectable objects that don't require external input, but with
+        no caching.
+        """
+        self.register_constructor(cls, None, caching_strategy=CacheStrategy.NO_CACHE)
         return cls
 
     def inject(self, func):
@@ -203,18 +227,28 @@ class InjectionManager:
         """
         @wraps(func)
         def wrapper(*args, **kwargs):
-            obj = args[0]
-            if hasattr(obj.__class__, '__annotations__'):
-                for name, val in inspect.getmembers(obj.__class__):
-                    if name[0:2] == "__":
-                        continue
-                    if name in obj.__class__.__annotations__:
-                        cls = obj.__class__.__annotations__[name]
-                        if self.cls_registry.is_injectable(cls):
-                            setattr(obj, name, self.context_manager.get_object(cls))
+            obj = args[0]  # self
+            type_map = self._get_bindable_attributes(obj.__class__)
+            for name, val in inspect.getmembers(obj.__class__):
+                if name[0:2] == "__":
+                    continue
+                if name in type_map and getattr(obj, name) is None:
+                    if self.cls_registry.is_injectable(type_map[name]):
+                        setattr(obj, name, self.context_manager.get_object(type_map[name]))
             return func(*args, **kwargs)
         return wrapper
 
+    def _get_bindable_attributes(self, cls: type) -> dict:
+        """Given a type, find all the bindable attributes using the annotations."""
+        type_map = {}
+        check_me = [cls, *cls.__mro__]
+        for check_cls in check_me:
+            if hasattr(check_cls, "__annotations__"):
+                for k in check_cls.__annotations__:
+                    if k not in type_map:
+                        type_map[k] = check_cls.__annotations__[k]
+        return type_map
+
     def _bind_parameters(self, func: callable, args: tuple, kwargs: dict):
         """ Inspects the given callable object and builds a new set of arguments for it with dependencies injected
 
